#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import facepy
from urlparse import parse_qs

import credentials

def connect(**kw):
  
  # parse kw's
  app_id = kw.get('app_id', credentials.FB_APP_ID)
  app_secret = kw.get('app_secret', credentials.FB_APP_SECRET)
  access_token = kw.get('access_token', None)

  # if no access token, create one
  if not access_token:
    access_token = generate_app_access_token(app_id, app_secret)

  # return api
  return facepy.GraphAPI(access_token)

def generate_app_access_token(app_id, app_secret):

  """
  Get an extended OAuth access token.

  :param access_token: A string describing an OAuth access token.
  :param application_id: An icdsnteger describing the Facebook application's ID.
  :param application_secret_key: A string describing the Facebook application's secret key.

  Returns a tuple with a string describing the extended access token and a datetime instance
  describing when it expires.
  """
  # access tokens
  default_access_token = facepy.get_application_access_token(
    application_id = app_id,  
    application_secret_key = app_secret
  )
  graph = facepy.GraphAPI(default_access_token)

  response = graph.get(
    path='oauth/access_token',
    client_id = app_id,
    client_secret = app_secret,
    grant_type = 'client_credentials'
  )
  components = parse_qs(response)
  token = components['access_token'][0]
  return token



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import datetime 
import pytz
import logging 
import time

from defaults import default_kws
from connection import connect
import facepy

try:
  import gevent
  from gevent.pool import Pool

  # patch everything except for thread.
  import gevent.monkey
  gevent.monkey.patch_socket()
  gevent.monkey.patch_ssl()
  gevent.monkey.patch_os()
  gevent.monkey.patch_time()
  gevent.monkey.patch_select()
  gevent.monkey.patch_subprocess()

except ImportError:
  imported_gevent = False
else:
  imported_gevent = True

FB_DATE_FORMAT = "%Y-%m-%dT%H:%M:%S+0000"

logging.basicConfig()
logger = logging.getLogger("zuckup")

def concurrent_yield(func, iterartor, **kw):
  if imported_gevent:
    p = Pool(kw.get('num_workers'))
    for result in p.imap_unordered(func, iterartor):
      yield result
  else:
    logger.warn('Cannot run concurrently without importing gevent')
    for args in iterartor:
      yield func(*args)

def utc_now():
  dt = datetime.utcnow()
  dt = dt.replace(tzinfo = pytz.utc)
  return dt

def strip_date(ds):
  dt = datetime.strptime(ds, FB_DATE_FORMAT)
  dt = dt.replace(tzinfo = pytz.utc)
  return dt

def validate_kw(kw, requires):
  """
  Validate kw.
  """
  # check required kwargs
  for r in requires:
    if r not in kw:
      raise Exception(
        "Missing required kwarg: %s" % r
        )

  # insert defaults 
  for k,v in default_kws.items():
    if k not in kw:
      kw[k] = v

  return kw

def opt_connect(**kw):
  """
  Connect to api if we havent passed a `conn` 
  argument in.
  """
  if 'conn' in kw:
    return conn 
  else:
    return connect(**kw)

def catch_err(func, api, **kw):
  """
  Catch Facebook API Errors, backoff, and timeout.
  """
  
  # get timing kwargs
  wait = kw.get('wait')
  backoff = kw.get('backoff')
  timeout = kw.get('timeout')

  # try until we timeout
  t0 = time.time()
  while True:
    try:
      results = func(api, **kw)
      break
    
    # backoff from errors
    except facepy.FacepyError as e:
      time.sleep(wait)
      wait *= backoff

      # timeout
      now = time.time()
      if now - t0 > timeout:
        logger.warn("Timing out beacause of {0}".format(e))
        results = []
        break

  return results


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from functools import wraps 

from parsers import (
  parse_insights, parse_post, parse_page_stats, get_datetime
  )
from util import (
  opt_connect, validate_kw, catch_err, concurrent_yield
  )

def fb(parser=parse_post,requires=[], default={}, gen=True):
  def fb_func(func, **kw):  
    @wraps(func)
    def f(*args, **kw):
      
      # check kw's
      kw = validate_kw(kw, requires)

      # optionally connect to the api
      api = opt_connect(**kw)

      # add defautls to kw's
      kw = dict(kw.items() + default.items())
        
      # get args
      args = catch_err(func, api, **kw)

      if gen:
        if kw.get('concurrent'):
          return concurrent_yield(parser, args, **kw)

        else:
          return (parser(a) for a in args)

      else:
        return parser(args)


    return f
  return fb_func

@fb(parser=parse_insights, requires=['page_id'])
def insights(api, **kw):
  
  # get the page id
  page_id = kw.get('page_id')
  
  # if paginate, double iterate
  if kw.get('page') or kw.get('paginate'):
    pages = api.get(page_id + "/posts", **kw)
    for page in pages:
      for post in page['data']:
        post_id = post['id']
        pub_datetime = get_datetime(post)
        insights = api.get(post_id + "/insights", **kw)
        yield insights, page_id, post_id, pub_datetime

  # if not, simple yield
  else:
    page = api.get(page_id + "/posts", **kw)
    for post in page['data']:
      post_id = post['id']
      pub_datetime = get_datetime(post)
      insights = api.get(post_id + "/insights", **kw)
      yield insights, page_id, post_id, pub_datetime


@fb(requires=['page_id'])
def page(api, **kw):
  # get the page id
  page_id = kw.get('page_id')
  
  # if paginate, double iterate
  if kw.get('page'):
    pages = api.get(page_id + "/posts", **kw)
    for page in pages:
      for post in page['data']:
        yield post, page_id

  # if not, simple yield
  else:
    page = api.get(page_id + "/posts", **kw)
    for post in page['data']:
      yield post, page_id


@fb(parser=parse_page_stats, requires=['page_id'], gen=False)
def page_stats(api, **kw):
  page_id = kw.get('page_id')
  page = api.get(page_id, **kw)
  return page, page_id

if __name__ == '__main__':
  for p in page(page_id="nytimes", concurrent=True):
    print p





#!/usr/bin/env python
# -*- coding: utf-8 -*-

default_kws = {
  'page' : False,
  'paginate' : True,
  'concurrent': False,
  'num_workers': 20,
  'since_id': None,
  'throttle' : 1,
  'limit' : 100,
  'wait': 1,
  'backoff': 2,
  'timeout': 30 
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from util import utc_now, strip_date

from siegfried import (
  prepare_url, is_short_url, unshorten_url, urls_from_string
  )

def parse_post(args):
  post, page_id = args
  data = {
    'page_id' : page_id,
    'post_id' : post.get('id', None),
    'urls' : get_urls(post),
    'img_url' : get_img(post),
    'datetime' : get_datetime(post),
    'message' : post.get('message', None),
    'description' : post.get('description', None),
    'status_type' : post.get('status_type', None),
    'type' : post.get('type', None)
  }
  return data

def parse_page_stats(args):
  page, page_id = args
  data = {
    'page_id' : page_id,
    'page_talking_about_count' : int(page['talking_about_count']),
    'page_likes' : int(page['likes']),
    'datetime' : utc_now()
  }
  return data

def parse_insights(args):
  """
  Get insights data if indicated so by the config file
  """
  insights, page_id, post_id, pub_datetime = args
  # add metadata
  insights = {
    'page_id' : page_id,
    'post_id' : post_id,
    'datetime' : utc_now(),
    'pub_datetime' : pub_datetime
  }

  # flatten dict
  for d in data:
    val = d['values'][0]['value']
    if isinstance(val, dict):
      for k, v in val.iteritems():
        insights[k] = v

    else:
      insights[d['name']] = val

  return insights

def get_url_candidates(post):
  
  urls = set()
  
  if post.has_key('link'):
    urls.add(post['link'])

  if post.has_key('source'):
    urls.add(post['source'])

  if post.has_key('message'):
    msg_urls = parse_message_urls(post['message'])
    for u in msg_urls: urls.add(u)

  return list(urls)

def parse_message_urls(message):
  urls = urls_from_string(message)
  return urls

# TODO: improve this
def get_img(post):
  return post.get('picture', None)

def get_urls(post):
  candidates = get_url_candidates(post)
  urls = set()
  for u in candidates:
    if 'facebook' not in u:
      if is_short_url(u):
        u = unshorten_url(u)
      urls.add(prepare_url(u))
  return list(urls)

def get_datetime(post):
  if post.has_key('updated_time'):
    return strip_date(post['updated_time'])
  else:
    return utc_now()






from setuptools import setup, find_packages
import os 

# hack for working with pandocs
import codecs 
try: 
  codecs.lookup('mbcs') 
except LookupError: 
  utf8 = codecs.lookup('utf-8') 
  func = lambda name, enc=utf8: {True: enc}.get(name=='mbcs') 
  codecs.register(func) 

# install readme
readme = os.path.join(os.path.dirname(__file__), 'README.md')

try:
  import pypandoc
  long_description = pypandoc.convert(readme, 'rst', format="md")
except (IOError, ImportError):
  long_description = ""

# setup
setup(
  name='zuckup',
  version='0.0.8',
  description='simple facebook parsing',
  long_description = long_description,
  classifiers=[
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    ],
  keywords='',
  author='Brian Abelson',
  author_email='brian@newslynx.org',
  url='http://github.com/newslynx/zuckup',
  license='MIT',
  packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
  namespace_packages=[],
  include_package_data=False,
  zip_safe=False,
  install_requires=[
    'facepy',
    'siegfried',
    'pytz'
  ],
  tests_require=[]
)